static int is_it_ms_file(char *path)
{
	CMSG_SIGNER_INFO *p_msg;
	CERT_INFO         cinf = {0};
	PCCERT_CONTEXT    p_cert;
	wchar_t           w_name[MAX_PATH];
	char              i_name[MAX_PATH];
	int               is_ms = 0;
	int               succs;
	u_long            encoding, msg_size;
	u_long            ct_type, fm_type;
	HCERTSTORE        h_store;
	HCRYPTMSG         h_msg;	

	mbstowcs(w_name, path, -1);

	h_store = NULL; h_msg = NULL; p_msg = NULL; p_cert = NULL;
	do
	{
		succs = CryptQueryObject(
			CERT_QUERY_OBJECT_FILE, w_name, CERT_QUERY_CONTENT_FLAG_PKCS7_SIGNED_EMBED, CERT_QUERY_FORMAT_FLAG_BINARY, 
			0, &encoding, &ct_type, &fm_type, &h_store, &h_msg, NULL); 

		if (succs == 0) {
			break;
		}

		succs = CryptMsgGetParam(
			h_msg, CMSG_SIGNER_INFO_PARAM, 0, NULL, &msg_size); 

		if (succs == 0) {
			break;
		}

		if ( (p_msg = malloc(msg_size)) == NULL ) {
			break;
		}

		succs = CryptMsgGetParam(
			h_msg, CMSG_SIGNER_INFO_PARAM, 0, p_msg, &msg_size); 

		if (succs == 0) {
			break;
		}

		cinf.Issuer       = p_msg->Issuer;
		cinf.SerialNumber = p_msg->SerialNumber;

		p_cert = CertFindCertificateInStore(
			h_store, X509_ASN_ENCODING | PKCS_7_ASN_ENCODING, 0, CERT_FIND_SUBJECT_CERT, &cinf, NULL);

		if (p_cert == NULL) {
			break;
		}

		succs = CertGetNameString(
			p_cert, CERT_NAME_SIMPLE_DISPLAY_TYPE, CERT_NAME_ISSUER_FLAG, NULL, i_name, sizeof(i_name));

		if (succs == 0) {
			break;
		}

		is_ms = (strcmp(i_name, "Microsoft Code Signing PCA") == 0);
	} while (0);

	if (p_cert != NULL) {
		CertFreeCertificateContext(p_cert);
	}

	if (p_msg != NULL) {
		free(p_msg);
	}

	if (h_msg != NULL) {
		CryptMsgClose(h_msg);
	}

	if (h_store != NULL) {
		CertCloseStore(h_store, 0);
	}

	return is_ms;
}

static int is_valid_sign(char *path)
{
	WINTRUST_FILE_INFO wt_fi = {0};
	WINTRUST_DATA      wt_data = {0};
	wchar_t            w_name[MAX_PATH];
	GUID               wt_guid = WINTRUST_ACTION_GENERIC_VERIFY_V2;
	u_long             status;

	mbstowcs(w_name, path, -1);

	wt_fi.cbStruct       = sizeof(WINTRUST_FILE_INFO);
    wt_fi.pcwszFilePath  = w_name;
    wt_fi.hFile          = NULL;
    wt_fi.pgKnownSubject = NULL;

	wt_data.cbStruct            = sizeof(wt_data);
    wt_data.pPolicyCallbackData = NULL;
    wt_data.pSIPClientData      = NULL;
    wt_data.dwUIChoice          = WTD_UI_NONE;
    wt_data.fdwRevocationChecks = WTD_REVOKE_NONE;
    wt_data.dwUnionChoice       = WTD_CHOICE_FILE;
    wt_data.dwStateAction       = 0;
    wt_data.hWVTStateData       = NULL;
    wt_data.pwszURLReference    = NULL;
    wt_data.dwProvFlags         = WTD_SAFER_FLAG | WTD_REVOCATION_CHECK_NONE;
    wt_data.dwUIContext         = 0;
    wt_data.pFile               = &wt_fi;

	status = WinVerifyTrust(
		NULL, &wt_guid, &wt_data);
	
	return (status == ERROR_SUCCESS);	
}

static int is_valid_update(char *path)
{
	if (is_it_ms_file(path) == 0) {
		return 0;
	}

	if (is_valid_sign(path) == 0) {
		return 0;
	}

	return 1;
}